# -*- coding: utf-8 -*-

import xbmcgui
import common


class ControlID:
    labelTitle = 341
    btExit = 342
    btOk = 343
    list = 141


def open_category_window(categories):
    window = CategoryWindow("smarty.tv.category.xml", common.get_addon_path(),
                            common.AddonSettings.get_skin_name())
    window.init(categories, common.KODI_ADDON)
    category = window.categorySelected
    del window
    return category


# noinspection PyPep8Naming
class CategoryWindow(xbmcgui.WindowXMLDialog):
    CATEGORY_ALL = -1
    NO_SELECTED = -2

    def __init__(self, *args, **kwargs):
        super(CategoryWindow, self).__init__(self, *args, **kwargs)

        self.addon = None
        self.categories = []
        self.title = u"Категории"

        self.categorySelected = self.NO_SELECTED

    def init(self, categories, addon):
        self.categories = categories
        self.addon = addon
        self.doModal()

    def onInit(self):
        common.log_info("Window open", "categoryWindow.onInit")

        super(CategoryWindow, self).onInit()

        title = self.getControl(ControlID.labelTitle)
        title.setLabel(self.title)

        self.fillCategoryList()

    def onCategorySelected(self):
        category_index = self.getSelectedCategory()
        self.categorySelected = category_index
        self.close()

    def onAction(self, action):
        controlID = self.getFocusId()

        if action == xbmcgui.ACTION_SELECT_ITEM or action == xbmcgui.ACTION_MOUSE_LEFT_CLICK \
                or action == xbmcgui.ACTION_MOUSE_DOUBLE_CLICK:
            if controlID == ControlID.btExit:
                common.log_info("Window close", "categoryWindow.onAction")
                self.close()
            elif controlID == ControlID.list or controlID == ControlID.btOk:
                self.onCategorySelected()

        xbmcgui.WindowXMLDialog.onAction(self, action)

    def setControlLabel(self, string, controlID):
        try:
            control = self.getControl(controlID)
            control.setLabel(string)
            common.log_debug("setting label, control %d" % controlID, "categoryWindow.setControlLabel")
        except:
            common.log_warning("error setting label, control %d" % controlID, "categoryWindow.setControlLabel")

    def getSelectedCategory(self):
        try:
            item = self.getControl(ControlID.list).getSelectedItem()
            category_index = int(item.getProperty('index'))
        except:
            category_index = self.NO_SELECTED
        return category_index

    def clearList(self):
        control = self.getControl(ControlID.list)
        control.reset()

    def fillCategoryList(self):
        def addCategoryItem(label, index=None):
            li = xbmcgui.ListItem()
            li.setLabel(label)
            if index is not None:
                li.setProperty('index', index)
            self.getControl(ControlID.list).addItem(li)

        self.clearList()

        addCategoryItem(common.get_localized_string(32100), str(-1))

        for cat in self.categories:
            colorizedName = common.colorize(cat.name, cat.color, True)
            index = str(cat.id)
            addCategoryItem(colorizedName, index)

        self.setFocusId(ControlID.list)
