import xbmc
import xbmcaddon
import xbmcgui
import config
import time
import locale
from datetime import date, timedelta, datetime

KODI_ADDON = xbmcaddon.Addon(id=config.ADDON_NAME)
KODI_MONITOR = xbmc.Monitor()


locale.setlocale(locale.LC_ALL, '')  # TODO: better internationalization


def get_localized_string(num):
    assert isinstance(num, int)
    return KODI_ADDON.getLocalizedString(num)


class GuiDialog(xbmcgui.Dialog):
    TYPE_OK = 0
    TYPE_YESNO = 1

    LEVEL_DEBUG = 0
    LEVEL_ERROR = 1
    LEVEL_NOTICE = 2

    HEADERS = [
        get_localized_string(34200),
        get_localized_string(34201),
        get_localized_string(34202),
    ]

    def __init__(self, message, level=LEVEL_NOTICE, type=TYPE_OK):
        self.level = level
        self.type = type
        self.message = message
        super(GuiDialog, self).__init__()

    def show(self):
        if self.type == GuiDialog.TYPE_YESNO:
            func = self.yesno
        else:
            func = self.ok
        header = GuiDialog.HEADERS[self.level]
        return func(header, self.message)


class AddonSettings:
    ABONEMENT = 'abon'
    ABONEMENT_OLD = 'abon_old'
    PASSWORD = 'password'
    PASSWORD_OLD = 'password_old'
    AUTHKEY = 'authkey'
    AUTOSTART_ENABLED = 'auto_start'
    TIME_OFFSET = 'time_offset'
    USE_SYSTEM_TIME_OFFSET = 'use_system_time_offset'


    @staticmethod
    def get_setting(setting):
        assert isinstance(setting, str)
        try:
            return KODI_ADDON.getSetting(setting)
        except:
            pass

    @staticmethod
    def get_info(setting):
        assert isinstance(setting, str)
        try:
            return KODI_ADDON.getAddonInfo(setting)
        except:
            pass

    @staticmethod
    def set_setting(setting, arg):
        assert isinstance(setting, str)
        assert isinstance(arg, str)
        try:
            KODI_ADDON.setSetting(setting, arg)
        except:
            pass

    @staticmethod
    def get_skin_name():
        # e = AddonSettings.get_setting('style')
        # if e == '0':
        #     return "Default"
        # elif e == '1':
        #     return "Impuls"
        return "Impuls"

    @staticmethod
    def check_setting_non_empty(setting):
        return AddonSettings.get_setting(setting) != ''

    @staticmethod
    def check_setting_is_true(setting):
        return AddonSettings.get_setting(setting) == 'true'

    @staticmethod
    def clear_setting(setting):
        AddonSettings.set_setting(setting, '')

    @staticmethod
    def is_auth():
        return AddonSettings.check_setting_non_empty(AddonSettings.AUTHKEY)


def open_settings():
    KODI_ADDON.openSettings()


def get_addon_info(info):
    return KODI_ADDON.getAddonInfo(info)


def get_addon_path():
    return get_addon_info('path')


def set_global_property(name, arg):
    try:
        str_arg = str(arg)
        xbmc.executebuiltin("SetProperty(%s, %s, 10000)" % (name, str_arg))
        log_info("Set global property %s - %s" % (name, str_arg),  "common.set_global_property")
    except:
        log_warning("Can't set global property %s" % name, "common.set_global_property")


def set_skin_property(name, arg):
    try:
        str_arg = str(arg)
        xbmc.executebuiltin("Skin.SetString(%s, %s)" % (name, str_arg))
        log_info("Set set property %s - %s" % (name, str_arg),  "common.set_skin_property")
    except:
        log_warning("Can't set skin property %s" % name, "common.set_skin_property")


def clear_property(name):
    xbmcgui.Window(10000).clearProperty(name)


def _log_format(message, module):
    return (u'[%s] %s : %s' % (config.ADDON_NAME, module, message)).encode('utf8')


def log_error(message, module):
    xbmc.log(_log_format(module, message), level=xbmc.LOGERROR)


def log_warning(message, module):
    xbmc.log(_log_format(module, message), level=xbmc.LOGWARNING)


def log_info(message, module):
    xbmc.log(_log_format(module, message), level=xbmc.LOGNOTICE)


def log_debug(message, module):
    xbmc.log(_log_format(module, message), level=xbmc.LOGDEBUG)


def get_time_offset():
    if AddonSettings.get_setting(AddonSettings.USE_SYSTEM_TIME_OFFSET) == 'false':
        try:
            return int(AddonSettings.get_setting(AddonSettings.TIME_OFFSET)) * 3600
        except:
            return 0
    return -time.timezone


def get_today_date(add_shift=False):
    current_time = time.time()
    if add_shift:
        current_time += get_time_offset()
    return date.fromtimestamp(current_time)


def get_today_string():
    return get_today_date().strftime("%d.%m.%Y")


def convert_date_to_str(date_to_convert):
    assert isinstance(date_to_convert, date)
    month_string_base = 33200
    weekday_string_base = 33100

    month = date_to_convert.month - 1
    weekday = date_to_convert.weekday()
    year = date_to_convert.year
    day = date_to_convert.day

    month_string = get_localized_string(month_string_base + month)
    weekday_string = get_localized_string(weekday_string_base + weekday)

    return u"%s, %d %s %d" % (weekday_string, day, month_string, year)


def convert_timestamp_to_str(timestamp, template='%H:%M'):
    timestamp += get_time_offset()
    dt = datetime.utcfromtimestamp(timestamp)
    return dt.strftime(template)


def get_now_time_str():
    return convert_timestamp_to_str(time.time())


def get_now_full_time_str():
    return convert_timestamp_to_str(time.time(), '%d %B %Y %H:%M')


def timestamp_into_percents(begin, current, end):
    if end < begin or not begin <= current <= end:
        raise Exception()
    # in range from 1 to 100
    percents = int(float(current - begin) / (end - begin) * 100) + 1
    return percents


def date_format(date):
    return date.strftime("%d %B")


def add_days(date, days):
    return date + timedelta(days=days)


def colorize(string, color, invert_black_and_white):
    if invert_black_and_white and color == '000000':
        color = 'FFFFFF'
    return u"[COLOR FF%s]%s[/COLOR]" % (color, string)


def close_splash_window(window):
    try:
        window.close()
        del window
    except:
        log_error("Can't close window", "common.close_splash_window")
