# -*- coding: utf-8 -*-

import time

import xbmcgui
import random

import common
import config
from common import AddonSettings
from epg_window import open_epg_window
from info_window import open_info_window
import category_window
from progress_loop import ProgressLoop, RefreshableTimer, Event
from smarty_api.user_api import UserApiClient, PortalSettingsField
from smarty_api.program_api import ProgramAPIClient
from smarty_api.channel_api import ChannelAPIClient
from settings import settings_instance
from splash_window import show_splash_window


class ControlID:
        imgBackground = 130
        imgRightBackground = 139

        btMenu = 242
        btCategory = 249
        btEPGList = 250
        btWindowList = 251

        imgLogo = 340
        labelTitle = 341
        labelTime = 342

        btPrev = 260
        btPause = 261
        btNext = 262
        btStop = 263
        btWindow = 264
        btEPG = 265
        btPin = 266
        labelBegin = 280
        labelEnd = 281

        labelChannel = 360
        labelProgram = 361
        imgChannel = 362
        labelDescription = 363
        labelCategories = 364
        grProgressBar = 162
        imgProgress = 461

        window = 140
        grChannels = 41
        listChannels = 141
        grCategories = 43
        listCategories = 143
        grMenu = 49
        listMenu = 149

        infoBar = 160
        infoBarBackground = 161
        labelStreamError = 170


# noinspection PyPep8Naming
class Overlay(xbmcgui.WindowXMLDialog):
    PAGE_CHANNELS = 0
    PAGE_MENU = 1
    PAGE_CATEGORIES = 2
    PAGE_PROFILE = 3

    MENU_ACTION_CHANNELS = 'channels'
    MENU_ACTION_CATEGORIES = 'categories'
    MENU_ACTION_INFO = 'info'
    MENU_ACTION_SETTINGS = 'settings'
    MENU_ACTION_BALANCE = 'balance'
    MENU_ACTION_LOGIN = 'login'
    MENU_ACTION_EXIT = 'exit'
    MENU_ACTION_LOGOUT = 'logout'
    MENU_ACTION_CATEGORY = 'category'
    MENU_ACTION_MENU = 'show_menu'
    MENU_ACTION_PROFILE = 'show_profile'

    CATEGORY_ALL = -1

    def __init__(self, *args, **kwargs):
        super(Overlay, self).__init__(self, *args, **kwargs)
        self._user_client = None
        self._program_client = None
        self._channel_client = None

        self._player = kwargs['player']
        self._player.stop()

        self.windowVisible = True

        self.infoBarCanBeVisible = False
        self.infoBarPinned = False
        self.infoBarHided = False
        self.infoBarReduced = False

        self.categoryOpened = False

        self.menuHideTimer = None
        self.nextRefreshTimer = None
        self.currentPlaylist = []

        self.progressBarLoop = None
        self.progressBarVisible = False
        self.progressBegin = 0
        self.progressEnd = 0

        self.page = Overlay.PAGE_CHANNELS
        self.category = Overlay.CATEGORY_ALL

        self.onAuthAction = None
        self.onLogoutAction = None
        self.onExitAction = None
        self.refreshAction = None
        self.onOverlayStarted = None
        self.onSettingsChanged = None

        self.menuButtonAction = self.MENU_ACTION_LOGIN

    # ==========================================
    # CALLBACKS
    # ==========================================

    # Do not uncomment unless you need to catch focus
    # def onFocus(self, controlId):
    #    if controlId == ControlID.listChannels:
    #        control = self.getControl(ControlID.listChannels)
    #        id = control.getSelectedPosition()
    #        while self.getFocusId() == ControlID.listChannels:
    #            xbmc.sleep(100)
    #            newId = control.getSelectedPosition()
    #            if id != newId:
    #                id = newId

    def onInit(self):
        super(Overlay, self).onInit()
        self.hideInfoBar()
        self.hideStreamError()
        self.setInfoBarVisibility(False)
        self.setWindowVisibility(True)

        common.set_skin_property('InfoBarPinned', 'false')

        self.setPageMenu()

        if self.onOverlayStarted:
            self.onOverlayStarted()

    def onAction(self, action):
        controlID = self.getFocusId()
        self.showInfoBar()

        buttonCode = action.getButtonCode()

        if buttonCode == 0xf041:  # 'a' button
            self._epgButtonAction()
        if buttonCode == 0xf057:  # 'w' button
            self.showCategoryWindow()
        elif action == xbmcgui.ACTION_CONTEXT_MENU:
            self._showGuiAction()
        elif action == xbmcgui.ACTION_NEXT_ITEM or action == xbmcgui.ACTION_CHANNEL_UP or \
             action == xbmcgui.ACTION_MOVE_UP and (self._player.isTVEnabled and not self.windowVisible):
            self._nextButtonAction()
        elif action == xbmcgui.ACTION_PREV_ITEM or action == xbmcgui.ACTION_CHANNEL_DOWN or \
             action == xbmcgui.ACTION_MOVE_DOWN and (self._player.isTVEnabled and not self.windowVisible):
            self._prevButtonAction()
        elif action == xbmcgui.ACTION_NAV_BACK:
            self._backAction()
        elif action == xbmcgui.ACTION_PREVIOUS_MENU:
            self._closeAction()
        elif action == xbmcgui.ACTION_SHOW_GUI:
            self._showGuiAction()
        elif action == xbmcgui.ACTION_STOP:
            self._stopButtonAction()
        elif action == 0x1A:
            self._epgButtonAction()
        elif action == xbmcgui.ACTION_SELECT_ITEM or action == xbmcgui.ACTION_MOUSE_LEFT_CLICK:
            if controlID == ControlID.btPrev:
                self._prevButtonAction()
            elif controlID == ControlID.btNext:
                self._nextButtonAction()
            elif controlID == ControlID.btStop:
                self._stopButtonAction()
            elif controlID == ControlID.btPause:
                self._pauseButtonAction()
            elif controlID == ControlID.btWindow:
                self._windowButtonAction()
            elif controlID == ControlID.btEPG or controlID == ControlID.btEPGList:
                self._epgButtonAction()
            elif controlID == ControlID.btPin:
                self._pinButtonAction()
            elif controlID == ControlID.btMenu:
                self._menuButtonAction()
            elif controlID == ControlID.btCategory:
                self.showCategoryWindow()
            elif controlID == ControlID.listChannels:
                self._mainListAction()
            elif controlID == ControlID.listCategories:
                self._categoryListAction()
            elif controlID == ControlID.listMenu:
                self._menuListAction()
            elif controlID == ControlID.btWindowList:
                self._showGuiAction()

    def onAuth(self):
        self._user_client = UserApiClient()
        self._program_client = ProgramAPIClient()
        self._channel_client = ChannelAPIClient()

        self._user_client.refresh_settings()
        self._channel_client.refresh_categories()
        self._channel_client.refresh_channels()

        # logoUrl = self._user_client.settings[PortalSettingsField.LOGO_URL]
        logoUrl = 'logo.url'
        common.AddonSettings.set_setting('logoUrl', logoUrl)

        self.setPageMenu()

        self.startProgressLoop()
        self.category = Overlay.CATEGORY_ALL
        self.setLogoImg()
        self.fillChannelsList(self.category, refresh=True)

        self.addChannelListRefreshEvent()
        self.addChannelRefreshEvent()

    def onLogout(self, isAuthChanged=False):
        self.stopProgressBarLoop()
        self._player.stopTV()
        self.clearControlList(ControlID.listChannels)
        self.setPageMenu()

    def onWindowVisibilityChanged(self):
        if self.windowVisible:
            self.setInfoBarPositionDown()
        else:
            self.setInfoBarPositionUp()
        self.setButtonMode()
        self.setDefaultFocus()

    # =========================================
    # CONTROL ACTIONS
    # =========================================

    def _loginButtonAction(self):
        # block buttons if window is not visible
        if not self.windowVisible:
            return

        if AddonSettings.is_auth():
            if self.onLogoutAction:
                self.onLogoutAction()
        else:
            if self.onAuthAction:
                self.onAuthAction()

    def _settingsButtonAction(self):
        # block buttons if window is not visible
        if not self.windowVisible:
            return
        self.openSettings()

    def _closeAction(self):
        if not self.windowVisible:
            return

        if self.page == Overlay.PAGE_MENU:
            self.close()
        else:
            self.setPageMenu()

    def _showGuiAction(self):
        if not self._player.isTVEnabled:
            return

        if self.windowVisible:
            self.setWindowVisibility(False)
        else:
            self.setWindowVisibility(True)

    def _backAction(self):
        if self.windowVisible:
            if self.page != Overlay.PAGE_MENU:
                self.setPageMenu()

    def _contextMenuAction(self):
        if self.windowVisible:
            if self.page == Overlay.PAGE_CHANNELS:
                self.showEPG(False)
        else:
            self.showEPG(True)

    def _prevButtonAction(self):
        if self._player.isArchiveMode:
            return

        if not self._player.isTVEnabled:
            common.log_warning("Try to switch channel, but TV is not started", "overlay.prevButtonAction")
            return
        self._player.prevChannel()

    def _nextButtonAction(self):
        if self._player.isArchiveMode:
            return
        if not self._player.isTVEnabled:
            common.log_warning("Try to switch channel, but TV is not started", "overlay.nextButtonAction")
            return
        self._player.nextChannel()

    def _pauseButtonAction(self):
        if not self._player.isPlayBackPaused():
            self._player.pause()
        else:
            self._player.resumePlayBack()

    def _mainListAction(self):
        # block list if window is not visible
        if not self.windowVisible:
            return
        item = self.getControl(ControlID.listChannels).getSelectedItem()
        if item is None:
            return

        index = int(item.getProperty('index'))
        self.playTV(index)

    def _categoryListAction(self):
        # block list if window is not visible
        if not self.windowVisible:
            return
        self.hideCategory()
        item = self.getControl(ControlID.listCategories).getSelectedItem()
        if item is None:
            return

        try:
            categoryID = int(item.getProperty('index'))
        except:
            common.log_error("Error getting category ID", "overlay.menuListAction")
            return
        self.setPageChannels(categoryID)

    def _menuListAction(self):
        # block list if window is not visible
        if not self.windowVisible:
            return
        item = self.getControl(ControlID.listMenu).getSelectedItem()
        if item is None:
            return

        action = item.getProperty('action')
        if action == Overlay.MENU_ACTION_CHANNELS:
            self.setPageChannels(Overlay.CATEGORY_ALL)
        elif action == Overlay.MENU_ACTION_INFO:
            self.showInfoWindow()
        elif action == Overlay.MENU_ACTION_LOGOUT or action == Overlay.MENU_ACTION_LOGIN:
            self._loginButtonAction()
        elif action == Overlay.MENU_ACTION_EXIT:
            self._exitButtonAction()
        elif action == Overlay.MENU_ACTION_SETTINGS:
            self.openSettings()
        elif action == Overlay.MENU_ACTION_BALANCE:
            self.showBalanceWindow()
        elif action == Overlay.MENU_ACTION_CATEGORIES:
            self.setPageCategories()
        elif action == Overlay.MENU_ACTION_PROFILE:
            self.setPageProfile()
        elif action == Overlay.MENU_ACTION_CATEGORY:
            try:
                categoryID = int(item.getProperty('index'))
            except:
                common.log_error("Error getting category ID", "overlay.menuListAction")
                return
            self.setPageChannels(categoryID)

    def _exitButtonAction(self):
        self.close()

    def _epgButtonAction(self):
        if self.windowVisible and self.page == Overlay.PAGE_CHANNELS:
            self.showEPG(False)
        elif self._player.isTVEnabled:
            self.showEPG(True)

    def _pinButtonAction(self):
        self.setInfoBarPinned()

    def _menuButtonAction(self):
        if self.menuButtonAction == self.MENU_ACTION_MENU:
            if not self.page == Overlay.PAGE_MENU:
                self.setPageMenu()
        elif self.menuButtonAction == self.MENU_ACTION_LOGIN:
            if not AddonSettings.is_auth() and self.onAuthAction:
                self.onAuthAction()
        elif self.menuButtonAction == self.MENU_ACTION_LOGOUT:
            if AddonSettings.is_auth() and self.onLogoutAction:
                self.onLogoutAction()

    def _windowButtonAction(self):
        self.setWindowVisibility(not self.windowVisible)

    def _stopButtonAction(self):
        self._player.stopTV()

    # =========================================
    # PLAYER ACTIONS
    # =========================================

    def pausedAction(self):
        pass

    def resumedAction(self):
        pass

    def startedAction(self):
        self.setWindowVisibility(False)
        self.setInfoBarVisibility(True)
        self.setDefaultFocus()

    def stoppedAction(self):
        self.setWindowVisibility(True)
        self.setInfoBarVisibility(False)
        self.setInfoBarPositionUp()
        self.hideInfoBar()
        self.clearProgramTime()
        self.setBackgroundVisibility(True)

    def channelChangeStartAction(self):
        self.setBackgroundVisibility(True)

        self.setInfoBarVisibility(True)
        self.showInfoBar()

        channel = self._channel_client.get_channel_by_id(self._player.currentChannelID)
        program = channel.getProgramByTime(time.time())

        self.setCurrentChannelTime(channel)
        self.fillInfoBar(channel, program)
        self.hideMenuTimerUpdate()

    def channelChangeEndsAction(self):
        self.setBackgroundVisibility(False)
        self.showInfoBar()
        self.hideMenuTimerUpdate()

    def archiveModeEnabledAction(self):
        pass

    def archiveTimeSetAction(self):
        if not self._player.isArchiveMode:
            return

        channelId = self._player.currentChannelID
        archiveTime = time.time() + self._player.archiveTimeDelta
        common.log_debug("archive time set action, channel id %d, time %d" %
                         (channelId, archiveTime), "overlay.archiveTimeSetAction")

        channel = self._channel_client.get_channel_by_id(channelId)
        program = self._program_client.get_program_by_time(channelId, archiveTime)
        self.fillInfoBar(channel, program)
        self.setCurrentProgramTime(program)

    # =========================================
    # COMMON GUI FUNCTIONS
    # =========================================

    def setCursorControl(self, controlID):
        control = self.getControl(controlID)
        btMenu = self.getControl(ControlID.btMenu)

        btMenu.setNavigation(control, control, control, control)

    def setVisibility(self, visibility, control):
        common.log_debug("visibility changed, control %d" % control, "overlay.setVisibility")
        window = self.getControl(control)
        window.setVisible(visibility)

    def hideMenuTimerUpdate(self):
        if self.infoBarPinned or not self.infoBarCanBeVisible:
            return
        hideTime = config.INFO_BAR_TIME
        if self.menuHideTimer is not None and self.menuHideTimer.started:
            self.menuHideTimer.refreshTime(hideTime)
        else:
            self.menuHideTimer = RefreshableTimer(self.hideInfoBar, hideTime)
            self.menuHideTimer.start()

    def showStreamError(self, errorMessage):
        self.setLabel(errorMessage, ControlID.labelStreamError)
        self.setVisibility(True, ControlID.labelStreamError)

    def hideStreamError(self):
        self.setVisibility(False, ControlID.labelStreamError)

    def setImg(self, url, control):
        try:
            img = self.getControl(control)
            img.setImage(url)
        except Exception as e:
            common.log_error(e.message, "overlay.setImage")

    def setLabel(self, text, control):
        try:
            label = self.getControl(control)
            label.setLabel(text)
        except Exception as e:
            common.log_error(e.message, "overlay.setLabel")

    def setTitle(self, text):
        self.setLabel(text, ControlID.labelTitle)

    def setControlEnabled(self, controlID, enabled):
        common.log_debug("Enabled changed, control %d" % controlID, "overlay.setControlEnabled")
        window = self.getControl(controlID)
        window.setEnabled(enabled)

    def getPageControlID(self):
        if self.page == Overlay.PAGE_CHANNELS:
            return ControlID.listChannels
        elif self.page == Overlay.PAGE_MENU or self.page == Overlay.PAGE_CATEGORIES:
            return ControlID.listMenu

    def setMenuButtonAction(self, action):
        self.menuButtonAction = action

    # ========================================
    # WINDOW
    # ========================================

    def close(self):
        dialog = common.GuiDialog(common.get_localized_string(32401), level=common.GuiDialog.LEVEL_NOTICE,
                                  type=common.GuiDialog.TYPE_YESNO)
        if not dialog.show():
            return

        if self.onExitAction:
            self.onExitAction()

        if self._player.isTVEnabled:
            self._player.stop()

        common.log_info("Add-on is closing, stop threads...", "overlay.close")
        if self.progressBarLoop is not None and self.progressBarLoop.started:
            self.progressBarLoop.stop()
        self.infoBarCanBeVisible = False
        if self.menuHideTimer is not None:
            self.menuHideTimer.stop()
        common.set_skin_property('MenuVisible', 'false')  # fixes some bug with info bar on the plugin restarting
        super(Overlay, self).close()

    def refreshNowTime(self):
        time = common.get_now_full_time_str()
        self.setLabel(time, ControlID.labelTime)

    def setBackgroundVisibility(self, visible):
        self.setVisibility(visible, ControlID.imgBackground)

    def setWindowVisibility(self, visible):
        self.windowVisible = visible
        self.setVisibility(visible, ControlID.window)
        self.onWindowVisibilityChanged()

    def setLogoImg(self):
        if self._user_client.settings and PortalSettingsField.LOGO_URL in self._user_client.settings:
            url = self._user_client.settings[PortalSettingsField.LOGO_URL]
        else:
            url = config.DEFAULT_LOGO_FILENAME

        common.log_debug("setting logo image: %s" % url, "overlay.setLogoImage")
        img = self.getControl(ControlID.imgLogo)
        img.setImage(url)

    def clearControlList(self, control):
        cList = self.getControl(control)
        cList.reset()

    def openSettings(self):
        common.open_settings()
        if self.onSettingsChanged:
            self.onSettingsChanged()

    def setChannelsList(self):
        grMenu = self.getControl(ControlID.grMenu)
        grChannels = self.getControl(ControlID.grChannels)
        # constant for 720p
        grMenu.setPosition(30, -1000)
        grChannels.setPosition(30, 80)
        self.setVisibility(False, ControlID.imgRightBackground)

        listControl = self.getControl(ControlID.listChannels)
        btMenu = self.getControl(ControlID.btMenu)
        btCategory = self.getControl(ControlID.btCategory)
        btEPGList = self.getControl(ControlID.btEPGList)

        btMenu.setNavigation(listControl, listControl, listControl, btCategory)
        btEPGList.setNavigation(listControl, listControl, btCategory, listControl)

    def setMenuList(self):
        grMenu = self.getControl(ControlID.grMenu)
        grChannels = self.getControl(ControlID.grChannels)
        # constant for 720p
        grChannels.setPosition(30, -1000)
        grMenu.setPosition(30, 80)
        self.setVisibility(True, ControlID.imgRightBackground)

        listControl = self.getControl(ControlID.listMenu)
        btMenu = self.getControl(ControlID.btMenu)
        btEPGList = self.getControl(ControlID.btEPGList)

        btMenu.setNavigation(listControl, listControl, listControl, listControl)
        btEPGList.setNavigation(listControl, listControl, listControl, listControl)

    def fitListHeight(self, controlID):
        """
        little hack
        prevents click handling
        """
        control = self.getControl(controlID)

        maxHeight = 540  # default, 720p
        if controlID == ControlID.listMenu:
            itemHeight = 90  # default, 720p
        else:
            itemHeight = 45  # default, 720p

        size = control.size()
        realHeight = itemHeight * size
        height = min(realHeight, maxHeight)
        control.setHeight(height)

    # =========================================
    # INFO BAR
    # =========================================

    def setInfoBarPinned(self):
        self.infoBarPinned = not self.infoBarPinned
        common.set_skin_property('InfoBarPinned', 'true' if self.infoBarPinned else 'false')

    def hideInfoBar(self):
        if self.infoBarHided or self.infoBarReduced or (self.infoBarCanBeVisible and self.infoBarPinned):
            return

        if not self.infoBarHided:
            common.set_skin_property('MenuVisible', 'false')
            self.infoBarHided = True

    def showInfoBar(self):
        if not self.infoBarCanBeVisible:
            return

        if self.infoBarHided:
            common.set_skin_property('MenuVisible', 'true')
            self.infoBarHided = False
        else:
            self.hideMenuTimerUpdate()

    def setInfoBarVisibility(self, visible):
        # visibility is not the same as hide
        self.infoBarCanBeVisible = visible

    def setInfoBarProgress(self, percents):
        if percents < 0:
            # progressbar behavior is incorrect on 0%
            percents = 1
        progress = self.getControl(ControlID.grProgressBar)
        progress.setPercent(percents)

    def setInfoBarPositionUp(self):
        menu = self.getControl(ControlID.infoBar)
        self.infoBarReduced = False
        # constant for 720p
        menu.setPosition(0, 600)

    def setInfoBarPositionDown(self):
        menu = self.getControl(ControlID.infoBar)
        self.infoBarReduced = True
        # constant for 720p
        menu.setPosition(0, 670)

    def fillInfoBar(self, channel, program):

        if program:
            beginStr = common.convert_timestamp_to_str(program.beginTime)
            endStr = common.convert_timestamp_to_str(program.endTime)
            programStr = program.name

            self.setLabel(beginStr, ControlID.labelBegin)
            self.setLabel(endStr, ControlID.labelEnd)
            self.setLabel(programStr, ControlID.labelProgram)
            self.setLabel(program.description, ControlID.labelDescription)
        else:
            self.setLabel('', ControlID.labelDescription)
            self.setLabel('', ControlID.labelProgram)
            self.setLabel('', ControlID.labelBegin)
            self.setLabel('', ControlID.labelEnd)

        self.setLabel(channel.name, ControlID.labelChannel)
        self.setImg(channel.iconURL, ControlID.imgChannel)

    # =============================================
    # THREAD LOOP EVENTS
    # =============================================

    def addChannelRefreshEvent(self):
        if config.ADAPTIVE_REFRESH_TIME:
            gap_start, gap_end = self._channel_client.get_next_refresh_gap()
            next_refresh_time = random.randint(gap_start, gap_end)
            common.log_debug("Set next channel cache refresh time: %d, gap %d-%d, now %s" %
                             (next_refresh_time, gap_start, gap_end, int(time.time())), "overlay.addChannelRefreshEvent")
        else:
            next_refresh_time = int(time.time()) + 120
            common.log_debug("Set next channel cache refresh time: %d, now %s" %
                             (next_refresh_time, int(time.time())), "overlay.addChannelRefreshEvent")

        event = Event(next_refresh_time, self.onChannelsRefresh, 'channels')
        self.progressBarLoop.addEvent(event)

    def addChannelListRefreshEvent(self):
        next_refresh_time = max(self._channel_client.get_next_program_end_time(), time.time() + 2)
        common.log_debug("Set next channel list refresh time: %d, now %d" % (next_refresh_time, int(time.time())),
                         "overlay.addChannelListRefreshEvent")
        event = Event(next_refresh_time, self.onChannelsListRefresh, 'channelList')
        self.progressBarLoop.addEvent(event)

    def onChannelsRefresh(self, obj, args):
        self._channel_client.refresh_channels()
        self.addChannelRefreshEvent()

    def onChannelsListRefresh(self, obj, args):
        self.fillChannelsList(self.category, refresh=True)
        self.addChannelListRefreshEvent()

    # =============================================
    # THREAD LOOP
    # =============================================

    def onProgressLoopProgramEnds(self, obj, args):
        if self._player.isTVEnabled:
            channel = self._channel_client.get_channel_by_id(self._player.currentChannelID)
            if self._player.isArchiveMode:
                archiveTime = time.time() + self._player.archiveTimeDelta
                program = self._program_client.get_program_by_time(channel.id, archiveTime)
            else:
                program = channel.getProgramByTime(time.time())
            self.fillInfoBar(channel, program)
            self.setCurrentProgramTime(program)

    def onProgressLoopStopped(self, obj, args):
        common.log_info("Timer stopped", "overlay.onProgressBarTimerStopped")
        self.setVisibility(False, ControlID.grProgressBar)
        self.setInfoBarProgress(100)

    def onProgressTimeRefresh(self, obj, args):
        common.log_info("Time refresh", "overlay.onProgressTimeRefresh")
        if len(args) != 2:
            return
        begin = args[0]
        end = args[1]
        if begin == 0 or begin >= end:
            self.setVisibility(False, ControlID.grProgressBar)
        else:
            self.setVisibility(True, ControlID.grProgressBar)

    def onProgressLoopWorks(self, obj, args):
        if len(args) != 3:
            common.log_error("Invalid arguments", "overlay.onProgressLoopWorks")
            return
        self.refreshNowTime()
        begin = args[0]
        current = args[1]
        end = args[2]
        if begin == end:
            return
        try:
            percent = common.timestamp_into_percents(begin, current, end)
        except:
            common.log_error("Invalid arguments", "overlay.onProgressLoopWorks")
            return
        self.setInfoBarProgress(percent)

    def setNextRefreshTimer(self, nextRefresh):
        currentTime = time.time()
        common.log_info("Set timer on %d, (%d now)" % (nextRefresh, currentTime), "overlay.setNextRefreshTimer")
        if self.progressBarLoop is None:
            common.log_warning("Can't set refresh time, thread is not started", "overlay.setNextRefreshTimer")
            return
        self.progressBarLoop.setNextRefreshTime(nextRefresh)

    def setCurrentChannelTime(self, channel):
        program = channel.getProgramByTime(time.time())
        self.setCurrentProgramTime(program)

    def setCurrentProgramTime(self, program=None):
        if self.progressBarLoop is None:
            common.log_warning("Can't set program time, thread is not started", "overlay.setCurrentProgramTime")
            return

        if program:
            timeDelta = self._player.archiveTimeDelta - 60
            beginTime = program.beginTime - timeDelta
            endTime = program.endTime - timeDelta
            self.progressBarLoop.setTime(beginTime, endTime)
        else:
            self.progressBarLoop.clearTime()

    def clearProgramTime(self):
        if self.progressBarLoop is None:
            common.log_warning("Can't clear time, thread is not started", "overlay.clearProgramTime")
            return
        self.progressBarLoop.clearTime()

    def startProgressLoop(self):
        if self.progressBarLoop is not None and self.progressBarLoop.started:
            self.stopProgressBarLoop()
        self.progressBarLoop = ProgressLoop()
        self.progressBarLoop.setOnWorkTimeChanged(self.onProgressTimeRefresh)
        self.progressBarLoop.setLoopStopped(self.onProgressLoopStopped)
        self.progressBarLoop.setTimeEnds(self.onProgressLoopProgramEnds)
        self.progressBarLoop.setTimerWork(self.onProgressLoopWorks)
        self.progressBarLoop.setWorkEnabled(True)
        self.progressBarLoop.start()
        common.log_info("Progress loop started", "overlay.startProgressLoop")

    def stopProgressBarLoop(self):
        if self.progressBarLoop is not None and self.progressBarLoop.started:
            self.progressBarLoop.stop()
            common.log_info("Progress loop stopped", "overlay.stopProgressBarLoop")

    # ================================================== #
    # ANOTHER WINDOWS                                    #
    # ================================================== #

    def showEPG(self, currentChannel):
        # if not self._player.isTVEnabled:
        #    common.log_warning("Tried to show EPG, but with no channel", "overlay.showEPG")

        if not currentChannel:
            item = self.getControl(ControlID.listChannels).getSelectedItem()
            if item is None:
                return

            index = int(item.getProperty('index'))
            if index >= len(self.currentPlaylist):
                return

            channel_id = self.currentPlaylist[index][1]  # url/channel ID pair
        else:
            channel_id = self._player.currentChannelID
            if channel_id <= 0:
                common.log_warning("Bad channel ID: %s" % str(channel_id), "overlay.showEPG")
                return

        channel = self._channel_client.get_channel_by_id(channel_id)
        program = open_epg_window(channel)
        if program:
            self.playProgram(program)

    def showCategoryWindow(self):
        if not self.page == Overlay.PAGE_CHANNELS:
            return

        category = category_window.open_category_window(self._channel_client.categories)
        if category != category_window.CategoryWindow.NO_SELECTED:
            self.setPageChannels(category)

    def showInfoWindow(self):
        if self._user_client.refresh_customer_info() != 0 :
            dialog = common.GuiDialog(common.get_localized_string(32402), level=common.GuiDialog.LEVEL_ERROR)
            dialog.show()
            return

        name = settings_instance.user_first_name + u' ' + settings_instance.user_last_name
        version = common.KODI_ADDON.getAddonInfo('version')
        portal_version = self._user_client.get_setting('backend_version')
        if portal_version:
            portal_version = u"Smarty v. %s" % portal_version

        dlist = [
            (common.get_localized_string(32200), name),
            (common.get_localized_string(32202), settings_instance.ip_address),
            (common.get_localized_string(32207), version),
            (u"Версия портала", portal_version),
        ]
        if settings_instance.contract_number:
            dlist.append((common.get_localized_string(32208), settings_instance.contract_number))

        open_info_window(dlist)

    def showBalanceWindow(self):
        if self._user_client.refresh_customer_info() != 0:
            dialog = common.GuiDialog(common.get_localized_string(32402), level=common.GuiDialog.LEVEL_ERROR)
            dialog.show()
            return

        balance = self._user_client.customer_info['balance']
        tariffs = self._user_client.customer_info['tariffs']
        monthly_payment = self._user_client.customer_info['monthly_payment']
        current_debt_amount = self._user_client.customer_info['current_debt_amount']

        dlist = [
            (common.get_localized_string(32203), float(balance)),
            (common.get_localized_string(32205), float(monthly_payment)),
            (common.get_localized_string(32206), float(current_debt_amount)),
            ('', ''),
            (common.get_localized_string(32204), ''),
        ]

        for tariff in tariffs:
            dlist.append(('    ' + tariff['name'], float(tariff['price'])))
        open_info_window(dlist)

    def showMessageWOAction(self, message):
        return show_splash_window(message)

    # ================================================
    # PAGES
    # ================================================

    def setButtonMode(self):
        def setButtonEnabled(control, isEnabled):
            self.setVisibility(isEnabled, control)
            self.setControlEnabled(control, isEnabled)

        if self.page == Overlay.PAGE_MENU:
            setButtonEnabled(ControlID.btMenu, False)
        else:
            setButtonEnabled(ControlID.btMenu, True)

        if self.page == Overlay.PAGE_CHANNELS:
            setButtonEnabled(ControlID.btEPGList, True)
            setButtonEnabled(ControlID.btCategory, True)
        else:
            setButtonEnabled(ControlID.btEPGList, False)
            setButtonEnabled(ControlID.btCategory, False)

        if self._player.isTVEnabled:
            setButtonEnabled(ControlID.btWindowList, True)
        else:
            setButtonEnabled(ControlID.btWindowList, False)

    def setDefaultFocus(self):
        if not self.windowVisible:
            self.setFocusId(ControlID.btWindow)
        elif self.page == Overlay.PAGE_CHANNELS:
            self.setFocusId(ControlID.listChannels)
        else:
            self.setFocusId(ControlID.listMenu)

    def setEmptyPage(self):
        self.setMenuList()
        self.page = Overlay.PAGE_MENU
        self.clearControlList(ControlID.listMenu)
        self.setTitle("")
        self.setButtonMode()

    def setPageChannels(self, categoryID):
        self.setMenuButtonAction(self.MENU_ACTION_MENU)
        self.setChannelsList()
        self.category = categoryID
        self.page = Overlay.PAGE_CHANNELS
        self.setButtonMode()
        self.setCategoryTitle()
        self.clearControlList(ControlID.listChannels)

        self.fillChannelsList(categoryID, False)
        self.setDefaultFocus()

    def addMenuListItem(self, label, action, index=None, icon=''):
        li = xbmcgui.ListItem()
        li.setLabel(label)
        li.setProperty('action', action)
        li.setArt({'icon': icon})
        if index is not None:
            li.setProperty('index', index)
        self.getControl(ControlID.listMenu).addItem(li)

    def setPageMenu(self):
        self.setMenuList()
        self.page = Overlay.PAGE_MENU
        self.clearControlList(ControlID.listMenu)
        self.setTitle(common.get_localized_string(32108))
        self.setButtonMode()

        if AddonSettings.is_auth():
            self.addMenuListItem(common.get_localized_string(32104), Overlay.MENU_ACTION_CHANNELS,
                                 icon="mainmenu_tvchannels_icon.png")
            self.addMenuListItem(u"Мой профиль", Overlay.MENU_ACTION_PROFILE,
                                 icon="mainmenu_info_icon.png")
        else:
            self.addMenuListItem(u"Войти в учётную запись", Overlay.MENU_ACTION_LOGIN,
                                 icon="mainmenu_balance_icon.png")

        self.addMenuListItem(common.get_localized_string(32105), Overlay.MENU_ACTION_SETTINGS,
                             icon="mainmenu_settings_icon.png")

        self.addMenuListItem(common.get_localized_string(32107), Overlay.MENU_ACTION_EXIT,
                             icon="mainmenu_exit_icon.png")

        self.fitListHeight(ControlID.listMenu)
        self.setDefaultFocus()

    def setPageProfile(self):
        self.setMenuList()
        self.page = Overlay.PAGE_PROFILE
        self.clearControlList(ControlID.listMenu)
        self.setTitle(u"Мой профиль")
        self.setButtonMode()

        if AddonSettings.is_auth():
            if self._user_client and self._user_client.settings is not None:
                try:
                    show_info = int(self._user_client.settings.get(PortalSettingsField.SHOW_INFO, 0))
                except:
                    show_info = 0

                if show_info == 1:
                    self.addMenuListItem(common.get_localized_string(32109), Overlay.MENU_ACTION_INFO,
                                         icon="mainmenu_info_icon.png")
            self.addMenuListItem(common.get_localized_string(32203), Overlay.MENU_ACTION_BALANCE,
                                 icon="mainmenu_balance_icon.png")
            self.addMenuListItem(u"Сменить учётную запись", Overlay.MENU_ACTION_LOGOUT,
                                 icon="mainmenu_exit_icon.png")

        self.setMenuButtonAction(Overlay.MENU_ACTION_MENU)

        self.fitListHeight(ControlID.listMenu)
        self.setDefaultFocus()

    # =========================================
    # CHANNELS
    # =========================================

    def playProgram(self, program):
        common.log_info("Play program %d on channel %s, start time: %d" %
                        (program.id, str(program.channelId), program.beginTime), 'overlay.playProgram')
        url = self._program_client.get_program_template_url(program.channelId)
        startTime = program.beginTime + 60
        self._player.playArchive(url, startTime, program.channelId)

    def playTV(self, index):
        self._player.playTV(self.currentPlaylist, index)

    def getChannelsByCategory(self, categoryID):
        if not self._channel_client.channels:
            spl = show_splash_window(common.get_localized_string(32400))
            self._channel_client.refresh_channels()
            common.close_splash_window(spl)

        if categoryID < 0:
            channels = self._channel_client.channels
        else:
            category = self._channel_client.get_category_by_id(categoryID)
            if category is None:
                return []
            channels = self._channel_client.get_channels_by_category(categoryID)
        channels.sort(key=lambda x: x.order)
        return channels

    def setCategoryTitle(self):
        if self.category != Overlay.CATEGORY_ALL:
            category = self._channel_client.get_category_by_id(self.category)
            if category:
                self.setTitle(category.name)
                return
        self.setTitle(common.get_localized_string(32104))

    def getChannelListItem(self, channel):
        playlistIndex = str(len(self.currentPlaylist))
        if channel.category is not None:
            channelOrder = common.colorize(str(channel.order), channel.category.color, True)
        else:
            channelOrder = str(channel.order)
        channelIndex = str(channel.id)

        program = channel.getProgramByTime(time.time())
        if program:
            description = program.description
            programName = program.name

            beginStr = common.convert_timestamp_to_str(program.beginTime)
            endStr = common.convert_timestamp_to_str(program.endTime)
            timeStr = "%s-%s" % (beginStr, endStr)
        else:
            description = ''
            programName = ''
            timeStr = ''

        iconURL = channel.iconURL

        li = xbmcgui.ListItem()

        li.setLabel(channel.name)
        li.setLabel2(programName)
        li.setArt({'icon': iconURL})
        li.setProperty('item_type', 'channel')
        li.setProperty('description', description)
        li.setProperty('time', timeStr)
        li.setProperty('index', playlistIndex)
        li.setProperty('channel_order', channelOrder)
        li.setProperty('channel_number', channelIndex)
        return li

    def fillChannelsList(self, category, refresh=False):
        common.log_info("Fill channel list", "overlay.selfRefresh")
        channels = self.getChannelsByCategory(category)
        if len(channels) == 0:
            return False

        showList = self.getControl(ControlID.listChannels)
        if refresh:
            selected = showList.getSelectedPosition()  # returns -1 if the list is empty
        else:
            selected = -1
        showList.reset()

        self.currentPlaylist = []
        for channel in channels:
            li = self.getChannelListItem(channel)
            self.currentPlaylist.append((channel.streamURL, channel.id))
            showList.addItem(item=li)

        self.fitListHeight(ControlID.listChannels)
        if selected >= 0:
            showList.selectItem(selected)
        return True
