# -*- coding: utf-8 -*-

import os
import sys
import urllib
import urllib2
import json

import resources.lib.config
from resources.lib.common import log_info, log_debug, log_error, log_warning
from resources.lib.common import AddonSettings


class SmartyAPIURL:
    LOGIN = "login/"
    LOGOUT = "logout/"
    CHANNEL_LIST = "channel/list/"
    CHANNEL_URL = "channel/url/"
    CATEGORY_LIST = "category/list/"
    CHANNEL_ICON_LIST = "channel/icon/list/"
    PROGRAM_LIST = "program/list/"
    PROGRAM_URL = "program/url/"
    PROGRAM_CATEGORY_LIST = "program/category/list/"
    PROGRAM_CATEGORY_CHANNEL_LIST = "program/category/channel/list/"
    ACCOUNT_STATUS = "account/status/"
    PORTAL_SETTINGS = "portal/settings/list/"
    CUSTOMER_INFO = "customer/info/"


class User:
    def __init__(self, abonement=None, password=None, authkey=None):
        self.abonement = abonement
        self.password = password
        self.authkey = authkey

        self.device = ''
        self.device_uid = 'eebeb81a64722d6509393a3aaa900b36'
        self.client_id = 0
        self.api_key = ''

        self.firstname = ''
        self.lastname = ''
        self.ip_address = ''
        self.activation_days_left = -1
        self.tariffs_list = None
        self.account_id = -1

        self.last_channel = None
        self.account_sort = None


def load_user_settings():
    abon = AddonSettings.get_setting(AddonSettings.ABONEMENT)
    password = AddonSettings.get_setting(AddonSettings.PASSWORD)
    authkey = AddonSettings.get_setting(AddonSettings.AUTHKEY)
    return User(abon, password, authkey)


class BaseSmartyApiClient(object):
    REQUEST_TIME_OUT = 1000
    BAD_RESPONSE = 1001
    AUTH_DATA_IS_EMPTY = 1002
    NOT_AUTHORIZED = 1003
    AUTH_USER_NOT_FOUND = 1
    AUTH_AUTHKEY_GENERATION_ERROR = 3
    AUTH_APIKEY_CHECK_ERROR = 4
    SETTINGS_CLIENT_OR_DEVICE_NOT_FOUND = 1
    SETTINGS_CANT_GET_DEVICE_SETTINGS = 2

    def __init__(self, user=None):
        self._user = load_user_settings() if user is None else user
        self._auth_ok = self._user.authkey is not None
        self._user.device = resources.lib.config.DEVICE
        self._user.client_id = resources.lib.config.CLIENT_ID
        self._user.api_key = resources.lib.config.API_KEY

        osname = '%s %s' % (os.name, sys.platform)
        self._user_agent = "Kodi (XBMC) smarty plugin on %s" % osname

    @property
    def user_agent(self):
        return self._user_agent

    def _get_auth_parameters(self):
        """
        Returns dictionary of base auth parameters
        """
        params = dict()
        params.update()
        if self._user.device:
            params['device'] = self._user.device
        if self._user.device_uid:
            params['device_uid'] = self._user.device_uid
        if self._user.client_id:
            params['client_id'] = self._user.client_id
        if self._user.authkey:
            params['authkey'] = self._user.authkey
        if self._user.api_key:
            params['api_key'] = self._user.api_key
        return params

    def _make_url_with_args(self, url, args):
        """
        Wrapper on urllib.urlencode
        """
        auth_args = self._get_auth_parameters()
        auth_args.update(args)
        args_string = urllib.urlencode(auth_args)
        out = "%s%s?%s" % (resources.lib.config.IPTV_API, url, args_string)
        return out

    def _request_json(self, url):
        ua = self.user_agent

        http_params = {'User-agent': ua,
                       'Connection': 'Close',
                       'Accept': 'application/json, text/javascript, */*'}

        if resources.lib.config.DEBUG:
            log_info(url, "iptv._request_json")

        try:
            req = urllib2.Request(url, headers=http_params)
            response_string = urllib2.urlopen(req, timeout=5).read()
        except urllib2.HTTPError as e:
            log_error(str(e), "iptv._request_json")
            return {'error': e.code}
        except urllib2.URLError as e:
            # TODO: URLError is not only timeout error
            log_error(str(e), "iptv._request_json")
            return {'error': BaseSmartyApiClient.REQUEST_TIME_OUT}

        if resources.lib.config.DEBUG and len(response_string) < 1000:
            log_info(response_string, "iptv._request_json")

        try:
            response = json.loads(response_string)
        except Exception as e:
            log_error("Can't parse response: %s" % str(e), "iptv._request_json")
            response = {'error': BaseSmartyApiClient.BAD_RESPONSE}
        return response

    @property
    def user(self):
        return self._user

    @property
    def is_auth(self):
        return self._auth_ok
