import xbmc
import xbmcgui
import common
from time import time


class TVPlayer(xbmc.Player):
    def __init__(self):
        super(TVPlayer, self).__init__()
        self._isTvEnabled = False
        self._currentChannel = -1
        self._TvPlayList = []

        self.onPlayBackPausedAction = None
        self.onPlayBackResumedAction = None
        self.onPlayBackStartedAction = None
        self.onPlayBackStoppedAction = None

        self.onTVStartedAction = None
        self.onTVResumedAction = None
        self.onTVStoppedAction = None

        self.onArchiveEnabledAction = None
        self.onArchiveTimeSetAction = None
        self.onChannelChangedAction = None

        self._playbackStartedArchiveTime = 0
        self._archiveUrlTemplate = 0
        self._playbackStartedRealTime = 0
        self._isArchiveMode = False

    def isPlaybackActive(self):
        # WARNING: It is not working as it should, do not uncomment
        # try:
        #     self.getPlayingFile()
        # except:
        #     return False

        xbmc.sleep(1000)
        if self.isPlaying():
            return True
        return False

    def playTV(self, channels, index=0):
        self._isTvEnabled = True
        self._isArchiveMode = False
        self._TvPlayList = channels
        self.onTVStarted()
        self.changeChannel(index)

    def playArchive(self, urlTemplate, beginTime, channelId):
        common.log_info("Play archive: cid %d, time %d" % (channelId, beginTime), "TVplayer.playArchive")
        self.onArchiveEnabled()

        self._archiveUrlTemplate = urlTemplate
        self._currentChannel = channelId
        self._isArchiveMode = True
        self._isTvEnabled = True

        self._playbackStartedRealTime = int(time())
        self._playbackStartedArchiveTime = beginTime

        self.setArchiveTime(beginTime)

    def setArchiveTime(self, time):
        common.log_info("Set archive time %d" % time, "TVplayer.setArchiveTime")
        if not self._isArchiveMode:
            return
        self.onArchiveTimeSet(time)
        super(TVPlayer, self).play(self._archiveUrlTemplate % time)

    def isPlayBackPaused(self):
        paused = bool(xbmc.getCondVisibility("Player.Paused"))
        return paused

    def stopTV(self):
        super(TVPlayer, self).stop()
        self._isTvEnabled = False
        self.onTVStopped()

    def resumePlayBack(self):
        xbmc.sleep(10)
        if self.isPlayBackPaused():
            self.pause()

    def getPlaybackTime(self):
        if not self._isTvEnabled:
            return 0
        currentTime = time()

        if self._isArchiveMode:
            return self._playbackStartedArchiveTime + (currentTime - self._playbackStartedRealTime)
        return currentTime

    def changeChannel(self, index):
        if index < 0 or index > self.channelsCount or not self.isTVEnabled:
            raise Exception
        self._currentChannel = index
        self.onChannelChanged()

        listitem = xbmcgui.ListItem('')
        listitem.setInfo('video', {})
        super(TVPlayer, self).play(self._TvPlayList[index][0], listitem)

    def nextChannel(self):
        if not self.isTVEnabled:
            raise Exception
        index = self._currentChannel + 1
        if index >= self.channelsCount:
            index = 0
        self.changeChannel(index)

    def prevChannel(self):
        if not self.isTVEnabled:
            raise Exception
        index = self._currentChannel - 1
        if index < 0:
            index = self.channelsCount - 1
        self.changeChannel(index)

    def onPlayBackPaused(self):
        if self.onPlayBackPausedAction:
            self.onPlayBackPausedAction()

    def onTVStarted(self):
        common.log_debug("TV started", "TVplayer.onTVStarted")
        if self.onTVStartedAction:
            self.onTVStartedAction()

    def onArchiveEnabled(self):
        common.log_debug("Archive mode enabled", "TVplayer.onArchiveEnabled")
        if self.onArchiveEnabledAction:
            self.onArchiveEnabledAction()

    def onArchiveTimeSet(self, time):
        common.log_debug("Set time %d" % time, "TVplayer.onArchiveTimeSet")
        if self.onArchiveTimeSetAction:
            self.onArchiveTimeSetAction()

    def onPlayBackResumed(self):
        if self.onTVResumedAction:
            self.onTVResumedAction()

    def onPlayBackStarted(self):
        if self.onPlayBackStartedAction:
            self.onPlayBackStartedAction()

    def onPlayBackStopped(self):
        if self.onPlayBackStoppedAction:
            self.onPlayBackStoppedAction()

    def onChannelChanged(self):
        if self.onChannelChangedAction:
            self.onChannelChangedAction()

    def onTVStopped(self):
        if self.onTVStoppedAction:
            common.log_info("TV stopped", "TVplayer.onTVStopped")
            self.onTVStoppedAction()

    @property
    def archiveTimeDelta(self):
        return self._playbackStartedArchiveTime - self._playbackStartedRealTime

    @property
    def currentChannelID(self):
        # common.log_info("TV stopped %s, %s" % (str(self._isArchiveMode), str(self._currentChannel)), "!!!!!!!!!")
        if self._currentChannel < 0 or self._isArchiveMode:
            return self._currentChannel
        if self._currentChannel > len(self._TvPlayList):
            return -1
        return self._TvPlayList[self._currentChannel][1]

    @property
    def channelsCount(self):
        return len(self._TvPlayList)

    @property
    def isTVEnabled(self):
        return self._isTvEnabled

    @property
    def isArchiveMode(self):
        return self._isArchiveMode

    @property
    def archiveTimeDelta(self):
        if not self._isArchiveMode:
            return 0
        return self._playbackStartedArchiveTime - self._playbackStartedRealTime

